@@ -4,7 +4,7 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2.1 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -317,6 +317,27 @@ dzl_shortcut_theme_find_default_context (DzlShortcutTheme *self,
   return dzl_shortcut_theme_find_default_context_by_type (self, G_OBJECT_TYPE (widget));
 }
 
+DzlShortcutContext *
+_dzl_shortcut_theme_find_default_context_with_phase (DzlShortcutTheme *self,
+                                                     GtkWidget        *widget,
+                                                     DzlShortcutPhase  phase)
+{
+  g_autofree gchar *free_me = NULL;
+  const gchar *name;
+
+  g_return_val_if_fail (DZL_IS_SHORTCUT_THEME (self), NULL);
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+
+  name = G_OBJECT_TYPE_NAME (widget);
+
+  if ((phase & DZL_SHORTCUT_PHASE_BUBBLE) != 0)
+    name = free_me = g_strdup_printf ("%s:bubble", name);
+  else if ((phase & DZL_SHORTCUT_PHASE_CAPTURE) != 0)
+    name = free_me = g_strdup_printf ("%s:capture", name);
+
+  return dzl_shortcut_theme_find_context_by_name (self, name);
+}
+
 void
 dzl_shortcut_theme_add_context (DzlShortcutTheme   *self,
                                 DzlShortcutContext *context)
@@ -331,7 +352,7 @@ dzl_shortcut_theme_add_context (DzlShortcutTheme   *self,
 
   g_return_if_fail (name != NULL);
 
-  g_hash_table_insert (priv->contexts, g_strdup (name), g_object_ref (context));
+  g_hash_table_insert (priv->contexts, (gchar *)g_intern_string (name), g_object_ref (context));
 }
 
 DzlShortcutTheme *
@@ -426,6 +447,29 @@ dzl_shortcut_theme_set_parent_name (DzlShortcutTheme *self,
     }
 }
 
+const gchar *
+_dzl_shortcut_theme_lookup_action (DzlShortcutTheme       *self,
+                                   const DzlShortcutChord *chord)
+{
+  DzlShortcutThemePrivate *priv = dzl_shortcut_theme_get_instance_private (self);
+
+  g_return_val_if_fail (DZL_IS_SHORTCUT_THEME (self), NULL);
+  g_return_val_if_fail (chord != NULL, NULL);
+
+  if (priv->actions_table != NULL)
+    {
+      const gchar *action = NULL;
+      DzlShortcutMatch match;
+
+      match = dzl_shortcut_chord_table_lookup (priv->actions_table, chord, (gpointer *)&action);
+
+      if (match == DZL_SHORTCUT_MATCH_EQUAL)
+        return action;
+    }
+
+  return NULL;
+}
+
 void
 dzl_shortcut_theme_set_chord_for_action (DzlShortcutTheme       *self,
                                          const gchar            *detailed_action_name,
@@ -455,13 +499,17 @@ dzl_shortcut_theme_set_chord_for_action (DzlShortcutTheme       *self,
     phase = DZL_SHORTCUT_PHASE_BUBBLE | DZL_SHORTCUT_PHASE_GLOBAL;
 
   if (!g_hash_table_contains (priv->chains, detailed_action_name))
-  {
-    DzlShortcutClosureChain *chain;
+    {
+      DzlShortcutClosureChain *chain;
 
-    chain = dzl_shortcut_closure_chain_append_action_string (NULL, detailed_action_name);
-    chain->phase = phase;
-    g_hash_table_insert (priv->chains, (gchar *)detailed_action_name, chain);
-  }
+      chain = dzl_shortcut_closure_chain_append_action_string (NULL, detailed_action_name);
+
+      if (chain != NULL)
+        {
+          chain->phase = phase;
+          g_hash_table_insert (priv->chains, (gchar *)detailed_action_name, chain);
+        }
+    }
 }
 
 const DzlShortcutChord *
